@@ -1,6 +1,6 @@
 #' @title Sparse Plus Low-Rank Matrices
 #'
-#' @description An 'S4' class for efficient computation with sparse plus
+#' @description An "S4" class for efficient computation with sparse plus
 #' low-rank matrices. Stores sparse plus low-rank matrices
 #' (e.g. from matrix factorization or centering graphs)
 #' of the form \code{x + a \%*\% t(b)} for faster
@@ -11,14 +11,15 @@
 #' @slot b optional. a low-rank factor for \code{a \%*\% t(b)}. if \code{b} is not provided, a will be factorized using
 #' \code{\link[irlba]{irlba}} provided \code{factorize = TRUE}
 #'
-#' @param x as in 'Matrix'
-#' @param a as in 'Matrix'
-#' @param b as in 'Matrix'
-#' @param ... as in 'Matrix'
+#' @param x as in "Matrix"
+#' @param a as in "Matrix"
+#' @param b as in "Matrix"
+#' @param ... as in "Matrix"
 #' 
 #' @return splrMatrix object
 #'
-#' @seealso Methods are documented in \code{\link{splr}}.
+#' @seealso Methods are documented in \link{splrMatrix_method}.
+#'  Other relevant methods are \link{splr_sparse_plus_constant} and 
 #'
 #' @rdname splr_constructor
 #'
@@ -27,7 +28,7 @@
 #' @importClassesFrom Matrix sparseMatrix
 setClass("splrMatrix",
   slots = c(x = "sparseMatrix", a = "Matrix", b = "Matrix"),
-  contains = 'sparseMatrix')
+  contains = "sparseMatrix")
 
 #' @rdname splr_constructor
 #'
@@ -41,6 +42,7 @@ setGeneric(
   name = "splr",
   def = function(x, a = NULL, b = NULL, rank = NULL,
   dimnames = list(NULL, NULL), ...) {
+
   #
   # x+ab'
   # x is mxn
@@ -97,7 +99,7 @@ setMethod(
   }
 )
 
-#' Convert splr 'Matrix' to Sparse
+#' Convert splr "Matrix" to Sparse
 #'
 #' @param data splrMatrix
 #'
@@ -106,6 +108,7 @@ setMethod(
 #' See \code{\link[Matrix]{Matrix}}.
 #'
 #' @export
+#' @keywords internal
 splr_to_sparse <- function(data){
     data@x + Matrix(data@a, sparse = TRUE) %*% Matrix(t(data@b), sparse = TRUE)
 }
@@ -118,7 +121,7 @@ as.matrix.splrMatrix <- function(from,...)  {
 
 #' Add a constant to a splrMatrix object
 #'
-#' @param x splrMatrix object
+#' @param x sparse Matrix object
 #' @param a scalar
 #'
 #' @return new splrMatrix object x + a
@@ -145,26 +148,26 @@ setAs("splrMatrix", "character", as.character.splrMatrix)
 
 setAs("splrMatrix", "matrix", function(from) as.matrix.splrMatrix(from))
 
-#' @title 'SPLR' Methods
+#' @title "SPLR" Methods
 #'
 #' @description Methods for the splrMatrix class. Most behave like
 #' Matrix methods though things like output show the
 #' decomposition. Use as.matrix to see the computed
 #' dense matrix.
 #'
-#' @param x As in 'Matrix'
-#' @param ... As in 'Matrix'
-#' @param object As in 'Matrix'
-#' @param e1 As in 'Matrix'
-#' @param y As in 'Matrix'
-#' @param e2 As in 'Matrix'
-#' @param type As in 'Matrix'
-#' @param na.rm As in 'Matrix'
-#' @param dims As in 'Matrix'
-#' @param i As in 'Matrix'
-#' @param j As in 'Matrix'
-#' @param drop As in 'Matrix'
-#' @param value As in 'Matrix'
+#' @param x As in "Matrix"
+#' @param ... As in "Matrix"
+#' @param object As in "Matrix"
+#' @param e1 As in "Matrix"
+#' @param y As in "Matrix"
+#' @param e2 As in "Matrix"
+#' @param type As in "Matrix"
+#' @param na.rm As in "Matrix"
+#' @param dims As in "Matrix"
+#' @param i As in "Matrix"
+#' @param j As in "Matrix"
+#' @param drop As in "Matrix"
+#' @param value As in "Matrix"
 #'
 #'
 #' @keywords internal
@@ -173,7 +176,8 @@ setAs("splrMatrix", "matrix", function(from) as.matrix.splrMatrix(from))
 #'  Attempts are made such that the returned object is stored efficiently,
 #'  either as a splrMatrix or sparse Matrix.
 #'
-#' @rdname splr
+#' @aliases splrMatrix_method
+#' @rdname splrMatrix_method
 setMethod("show", signature("splrMatrix"),
   function(object){
     cat("Sparse part\n")
@@ -184,7 +188,7 @@ setMethod("show", signature("splrMatrix"),
     show(object@b)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("print", signature("splrMatrix"),
   function(x){
     cat("Sparse part\n")
@@ -213,9 +217,9 @@ setMethod("print", signature("splrMatrix"),
 
 
 
-# #' @rdname splr
+# #' @rdname splrMatrix_method
 # setMethod('-',
-#   signature(e1 = 'splrMatrix', e2 = 'missing'),
+#   signature(e1 = "splrMatrix", e2 = "missing"),
 #   function(e1, e2 = NULL) {
 #     new("splrMatrix", x = -e1@x, a = -e1@a, b = e1@b, Dim = dim(e1@x))
 #   })
@@ -239,7 +243,7 @@ setMethod("print", signature("splrMatrix"),
   }
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "splrMatrix", y = "splrMatrix"), function(x, y) {
   new("splrMatrix",
     x = x@x %*% y@x,
@@ -249,31 +253,31 @@ setMethod("%*%", signature(x = "splrMatrix", y = "splrMatrix"), function(x, y) {
 })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "splrMatrix", y = "matrix_list"),
   function(x, y){
     matrix_list(lapply(seq_along(y), function(i) x %*% y[[i]]))
   })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "matrix_list", y = "splrMatrix"),
   function(x, y){
     matrix_list(lapply(seq_along(x), function(i) x[[i]] %*% y))
   })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "Matrix", y = "splrMatrix"), .leftmult)
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "matrix", y = "splrMatrix"), .leftmult)
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "numeric", y = "splrMatrix"), .leftmult)
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x = "numLike", y = "splrMatrix"), .leftmult)
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="ANY", y ="splrMatrix"),.leftmult)
 
 .rightmult = function(x, y){
@@ -300,25 +304,25 @@ setMethod("%*%", signature(x ="ANY", y ="splrMatrix"),.leftmult)
 
 
 }
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("dim", signature(x = "splrMatrix"), function(x) { dim(x@x)})
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("length", signature(x = "splrMatrix"), function(x) { length(x@x)})
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="splrMatrix", y ="Matrix"),.rightmult)
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="splrMatrix", y ="matrix"),.rightmult)
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="splrMatrix", y ="numeric"),.rightmult)
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="splrMatrix", y ="numLike"),.rightmult)
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("%*%", signature(x ="splrMatrix", y ="ANY"),.rightmult)
 
 #doesn't return an splr
-#' @rdname splr
-setMethod('*', signature = signature(e1 = 'splrMatrix', e2 = 'splrMatrix'), function(e1, e2) {
+#' @rdname splrMatrix_method
+setMethod('*', signature = signature(e1 = "splrMatrix", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e1, e2)
 })
 
@@ -341,80 +345,80 @@ setMethod('*', signature = signature(e1 = 'splrMatrix', e2 = 'splrMatrix'), func
   }
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'Matrix', e2 = 'splrMatrix'), function(e1, e2) {
+  signature (e1 = "Matrix", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e2, e1)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'splrMatrix', e2 = 'ddiMatrix'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "ddiMatrix"), function(e1, e2) {
   .multiply(e1, e2)
 })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'ddiMatrix', e2 = 'splrMatrix'), function(e1, e2) {
+  signature (e1 = "ddiMatrix", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e2, e1)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'matrix', e2 = 'splrMatrix'), function(e1, e2) {
+  signature (e1 = "matrix", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e2, e1)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'numeric', e2 = 'splrMatrix'), function(e1, e2) {
+  signature (e1 = "numeric", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e2, e1)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'ANY', e2 = 'splrMatrix'), function(e1, e2) {
+  signature (e1 = "ANY", e2 = "splrMatrix"), function(e1, e2) {
   .multiply(e2, e1)
 })
 
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'splrMatrix', e2 = 'matrix'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "matrix"), function(e1, e2) {
   .multiply(e1, e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'splrMatrix', e2 = 'Matrix'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "Matrix"), function(e1, e2) {
   .multiply(e1, e2)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'splrMatrix', e2 = 'numeric'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "numeric"), function(e1, e2) {
   .multiply(e1, e2)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("*",
-  signature (e1 = 'splrMatrix', e2 = 'ANY'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "ANY"), function(e1, e2) {
   .multiply(e1, e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("/",
-  signature (e1 = 'splrMatrix', e2 = 'matrix'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "matrix"), function(e1, e2) {
   .multiply(e1, 1/e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("/",
-  signature (e1 = 'splrMatrix', e2 = 'Matrix'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "Matrix"), function(e1, e2) {
   .multiply(e1, 1/e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("/",
-  signature (e1 = 'splrMatrix', e2 = 'ANY'), function(e1, e2) {
+  signature (e1 = "splrMatrix", e2 = "ANY"), function(e1, e2) {
 
   .multiply(e1, 1/e2)
 })
@@ -423,10 +427,10 @@ setMethod("/",
   new("splrMatrix", x = as(e1@x + e2@x,"sparseMatrix"), a = cbind2(e1@a, e2@a), b = cbind2(e1@b, e2@b), Dim = dim(e1))
 }
 
-#' @rdname splr
-setMethod('+', signature = signature(e1 = 'splrMatrix', e2 = 'splrMatrix'),.addSplr)
-#' @rdname splr
-setMethod('-', signature = signature(e1 = 'splrMatrix', e2 = 'splrMatrix'), function(e1, e2) {
+#' @rdname splrMatrix_method
+setMethod('+', signature = signature(e1 = "splrMatrix", e2 = "splrMatrix"),.addSplr)
+#' @rdname splrMatrix_method
+setMethod('-', signature = signature(e1 = "splrMatrix", e2 = "splrMatrix"), function(e1, e2) {
   .addSplr(e1,-e2)
 })
 
@@ -447,69 +451,69 @@ setMethod('-', signature = signature(e1 = 'splrMatrix', e2 = 'splrMatrix'), func
   }
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature(e1 ="splrMatrix", e2 ="Matrix"), function(e1, e2) {
   .leftadd(e1 = e1, e2 = e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature(e1 ="splrMatrix", e2 ="numeric"), function(e1, e2) {
   .leftadd(e1 = e1, e2 = e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature(e1 ="splrMatrix", e2 ="ANY"), function(e1, e2) {
   .leftadd(e1 = e1, e2 = e2)
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature(e1 = "splrMatrix", e2 = "missing"),
   function(e1, e2 = NULL){
-    splr(-e1@x, a = -e1@a, b = e1@a)
+    splr(-e1@x, a = -e1@a, b = e1@b)
   })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature(e1 ="splrMatrix", e2 ="Matrix"),
   function(e1, e2) {
     .leftadd(e1 = e1, e2 = -e2)
   })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature(e1 ="splrMatrix", e2 ="ddiMatrix"),
   function(e1, e2) {
     .leftadd(e1 = e1, e2 = -e2)
   })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature(e1 ="splrMatrix", e2 ="numeric"), function(e1, e2) {
   .leftadd(e1 = e1, e2 = -e2)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature(e1 ="splrMatrix", e2 ="ANY"), function(e1, e2) {
   .leftadd(e1 = e1, e2 = -e2)
 })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature("Matrix","splrMatrix"), function(e1, e2) {
   .leftadd(e2, e1)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature("numeric","splrMatrix"), function(e1, e2) {
   .leftadd(e2, e1)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("+", signature("ANY","splrMatrix"), function(e1, e2) {
   .leftadd(e2, e1)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature("Matrix","splrMatrix"), function(e1, e2) {
   .leftadd(e2, e1)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature("numeric", "splrMatrix"), function(e1, e2) {
   .leftadd(-e2, e1)
 })
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("-", signature("ANY","splrMatrix"), function(e1, e2) {
   .leftadd(-e2, e1)
 })
@@ -521,7 +525,7 @@ setMethod("-", signature("ANY","splrMatrix"), function(e1, e2) {
 Frobsmlr = function(x, a, b){
 
     #expansion due to trevor hastie
-    xnorm <- norm(x, type = 'f')
+    xnorm <- norm(x, type = "f")
     xab = as.matrix(x%*%b)
     xab = sum(xab*a)
     aa = t(a)%*%a
@@ -534,7 +538,7 @@ Frobsmlr = function(x, a, b){
 }
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("norm", signature(x ="splrMatrix", type ="character"),
   function(x, type,...){
     switch(type,
@@ -549,9 +553,12 @@ setMethod("norm", signature(x ="splrMatrix", type ="character"),
 #' @param x matrix like object
 #' @param y matrix like object
 #'
-#' @returns inner product <x, y>
+#' @returns inner product <x, y> = sum over all elements i,j of x_ij * y_ij.
+#' 
+#' @details For a matrix_list object, sums over all layers/list-elements as well.
 #'
 #' @rdname innerproduct
+#' @keywords internal
 #' @export
 setGeneric("innerproduct", function(x, y){
   sum(x * y)
@@ -603,7 +610,7 @@ setMethod(
 
 
 }
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("rowSums",
   signature(x = "splrMatrix"),
   .rsum)
@@ -619,7 +626,7 @@ setMethod("rowSums",
 
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("colSums",
   signature(x = "splrMatrix"),
   .csum)
@@ -635,7 +642,7 @@ setMethod("colSums",
 
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("rowMeans",
   signature(x = "splrMatrix"),
   .rmean)
@@ -651,7 +658,7 @@ setMethod("rowMeans",
 
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("colMeans",
   signature(x = "splrMatrix"),
   .cmean)
@@ -661,27 +668,27 @@ setMethod("colMeans",
   sum(.csum(x), na.rm = na.rm)
 }
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("sum", signature(x = "splrMatrix", na.rm = "ANY"), .sum)
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("mean", signature(x = "splrMatrix"), function(x, ...){
   sum(x, ...) / x@Dim[1] / x@Dim[2]
 })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("[",
   signature(x = "splrMatrix",
-    i = 'missing', j = 'missing', drop = 'missing') ,
+    i = "missing", j = "missing", drop = "missing") ,
   function(x, i = NULL, j = NULL, drop = NULL) {
           x
   })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'numeric', drop = 'logical')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "numeric", drop = "logical")
           , function(x, i, j, ..., drop) {
             if (drop) {
-              warning("drop = TRUE is ignored for the splrMatrix class. cast to another clas first")
+              warning("drop = TRUE is ignored for the splrMatrix class. cast to another class first")
             }
             return(splr(x@x[i, j], x@a[i, ], x@b[j,]
                          , Dim = dim(x@x[i, j])) )
@@ -696,7 +703,7 @@ col_index <- function(x, j, ..., drop) {
   }
 
   if (drop) {
-   warning("drop = TRUE is ignored for the splrMatrix class. cast to another clas first")
+   warning("drop = TRUE is ignored for the splrMatrix class. cast to another class first")
   }
   return(new(
     "splrMatrix",
@@ -709,20 +716,20 @@ col_index <- function(x, j, ..., drop) {
 
 }
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'missing', j = 'numeric', drop = 'logical'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "missing", j = "numeric", drop = "logical"),
             col_index)
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'missing', j = 'numeric', drop = 'missing'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "missing", j = "numeric", drop = "missing"),
             function(x, j, ...) col_index(x, j, drop = FALSE))
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'missing', j = 'logical', drop = 'logical'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "missing", j = "logical", drop = "logical"),
             col_index)
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'missing', j = 'logical', drop = 'missing'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "missing", j = "logical", drop = "missing"),
             function(x, j, ...) col_index(x, j, drop = FALSE))
 
 row_index <- function(x, i, ..., drop) {
@@ -735,7 +742,7 @@ row_index <- function(x, i, ..., drop) {
 
 
   if (drop) {
-   warning("drop = TRUE is ignored for the splrMatrix class. cast to another clas first")
+   warning("drop = TRUE is ignored for the splrMatrix class. cast to another class first")
   }
   return( new("splrMatrix",
     x = x@x[i, j,...],
@@ -744,51 +751,51 @@ row_index <- function(x, i, ..., drop) {
     Dim = dim(x = x@x[i, j,...])))
 }
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'missing', drop = 'logical'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "missing", drop = "logical"),
             row_index)
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'missing', drop = 'missing'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "missing", drop = "missing"),
             function(x, i, ...) row_index(x, i, drop = FALSE))
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'missing', drop = 'logical'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "missing", drop = "logical"),
             row_index)
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'missing', drop = 'missing'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "missing", drop = "missing"),
             function(x, i, ...) row_index(x, i, drop = FALSE))
 
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'ANY', drop ='logical')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "ANY", drop ='logical')
           , function(x, i, j, ..., drop) {
             return(splr(x = x@x[i, j,..., drop = FALSE], a = x@a[i,, drop = FALSE], b = x@b[j,, drop = FALSE]
                        , Dim = dim(x@x[i, j,..., drop = FALSE])))
 
           })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'logical', drop ='logical')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "logical", drop ='logical')
           , function(x, i, j, ..., drop) {
             if (drop) {
-              warning("drop = TRUE is ignored for the splrMatrix class. cast to another clas first")
+              warning("drop = TRUE is ignored for the splrMatrix class. cast to another class first")
             }
             return(splr(x = x@x[i, j,..., drop = FALSE], a = x@a[i,, drop = FALSE], b = x@b[j,, drop = FALSE]
                        , Dim = dim(x@x[i, j,..., drop = FALSE])))
 
           })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'numeric', j = 'ANY', drop ='missing')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "numeric", j = "ANY", drop ='missing')
           , function(x, i, j, ..., drop = FALSE) {
             return(splr(x = x@x[i, j,..., drop = FALSE], a = x@a[i,, drop = FALSE], b = x@b[j,, drop = FALSE],
                        Dim = dim(x = x@x[i, j,..., drop = FALSE])))
   })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'ANY', drop = 'ANY')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "ANY", drop = "ANY")
           , function(x, i, j, ..., drop) {
             new('splrMatrix', x = x@x[i, j, drop = FALSE]
                 , a = as(x@a[i,, drop = FALSE],'Matrix'), b = as(x@b[j,, drop = FALSE],'Matrix')
@@ -798,8 +805,8 @@ setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'ANY', drop = 'ANY'
  })
 
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'ANY', drop = 'missing')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "ANY", drop = "missing")
           , function(x, i, j, ...) {
 
 
@@ -807,8 +814,8 @@ setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'ANY', drop = 'miss
                 , Dim = dim(as(x@x[i, j], "Matrix")))
     })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'numeric', drop = 'missing')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "numeric", drop = "missing")
           , function(x, i, j, ...) {
 
 
@@ -816,11 +823,11 @@ setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'numeric', drop = '
                 , Dim = dim(as(x@x[i, j], "Matrix")))
     })
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'numeric', drop = 'logical')
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "logical", j = "numeric", drop = "logical")
           , function(x, i, j, ..., drop) {
             if (drop) {
-              warning("drop = TRUE is ignored for the splrMatrix class. cast to another clas first")
+              warning("drop = TRUE is ignored for the splrMatrix class. cast to another class first")
             }
             ret <- new('splrMatrix', x = as(x@x[i, j], "Matrix"), a = x@a[i,, drop = FALSE], b = x@b[j,, drop = FALSE]
                 , Dim = dim(as(x@x[i, j], "Matrix")))
@@ -829,15 +836,15 @@ setMethod("[", signature(x ="splrMatrix", i = 'logical', j = 'numeric', drop = '
 
 
 
-#' @rdname splr
-setMethod("[", signature(x ="splrMatrix", i = 'missing', j = 'missing', drop = 'missing'),
+#' @rdname splrMatrix_method
+setMethod("[", signature(x ="splrMatrix", i = "missing", j = "missing", drop = "missing"),
   function(x, i = NULL, j = NULL, drop = NULL) {
     x
   })
 
 
 #can fix this
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("[",
     signature(x = "splrMatrix", i = "matrix", j = "missing", drop = "missing"),
     function(x, i , ...) {
@@ -850,27 +857,23 @@ setMethod("[",
 
 #document the issues with doing this
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("[<-",
-  signature(x ="splrMatrix", i = 'numeric', j = 'numeric', value = 'ANY'),
+  signature(x = "splrMatrix", i = "numeric", j = "numeric", value = "ANY"),
   function(x, i, j, ..., value) {
     y <- x@x
-    y[i, j] <- value
     a <- x@a
-    if (length(i) > 0) {
-      a[i,] <- 0
-    }
     b <- x@b
-    if (length(j) > 0) {
-      b[j,] <- 0
-    }
+    y[i, j] <- value -
+      tcrossprod(a[i, , drop=FALSE], b[j, , drop=FALSE])
+
     new("splrMatrix", x = y, a = a, b = b, Dim = dim(y))
 
-   })
+  })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("[<-",
-  signature(x ="splrMatrix", i = 'numeric', j = 'missing', value = 'ANY') ,
+  signature(x ="splrMatrix", i = "numeric", j = "missing", value = "ANY") ,
   function(x, i, ..., value) {
     j <- c(1:dim(x@x)[2])
     y <- x@x
@@ -878,19 +881,17 @@ setMethod("[<-",
     a <- x@a
     a[i,] <- 0
     b <- x@b
-    b[j,] <- 0
     new("splrMatrix", x = y, a = a, b = b, Dim = dim(y))
  })
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("[<-",
-  signature(x ="splrMatrix", i = 'missing', j = 'numeric', value = 'ANY') ,
+  signature(x ="splrMatrix", i = "missing", j = "numeric", value = "ANY") ,
   function(x, j, ..., value) {
     i <- c(1:dim(x@x)[1])
     y <- x@x
     y[, j] <- value
     a <- x@a
-    a[i,] <- 0
     b <- x@b
     b[j,] <- 0
     new("splrMatrix", x = y, a = a, b = b, Dim = dim(y))
@@ -898,33 +899,36 @@ setMethod("[<-",
 
 
 # Implementing this would be nice
-#' @rdname splr
-setMethod("[<-", signature(x ="Matrix", i = 'ANY', j = 'ANY', value = 'splrMatrix'),
-          function(x, i, j, ..., value) {
-            y <- x
-            y[i, j] <- value@x
-            a <- Matrix(0, dim(x)[1], dim(value@a)[2])
-            b <- Matrix(0, dim(x)[2], dim(value@b)[2])
-            a[i,] <- value@a
-            b[j,] <- value@b
-            new("splrMatrix", x = y, a = a, b = b, Dim = dim(y))
- })
+#' @rdname splrMatrix_method
+setMethod(
+  "[<-",
+  signature(x ="Matrix", i = "ANY", j = "ANY", value = "splrMatrix"),
+  function(x, i, j, ..., value) {
+    y <- x
+    y[i, j] <- value@x
+    a <- Matrix(0, dim(x)[1], dim(value@a)[2])
+    b <- Matrix(0, dim(x)[2], dim(value@b)[2])
+    a[i,] <- value@a
+    b[j,] <- value@b
+    new("splrMatrix", x = y, a = a, b = b, Dim = dim(y))
+  }
+)
 
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("dim", signature(x = "splrMatrix"),
           function(x) x@Dim, valueClass = "integer")
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod('str', signature(object = "splrMatrix"), function(object){
-  cat('splrMatrix')
-  cat('\nDimension: ', dim(object@x))
-  cat('\nLower rank matrix is rank: ', min(dim(object@a)) )
+  cat("splrMatrix")
+  cat("\nDimension: ", dim(object@x))
+  cat("\nLower rank matrix is rank: ", min(dim(object@a)) )
 })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("t", signature = signature(x ="splrMatrix") , function(x) {
   #splr(t(x@x), x@b, x@a)
 
@@ -932,7 +936,7 @@ setMethod("t", signature = signature(x ="splrMatrix") , function(x) {
 })
 
 
-#' @rdname splr
+#' @rdname splrMatrix_method
 setMethod("diag", signature = signature(x = "splrMatrix"), function(x) {
   drop(diag(x@x) + rowSums(x@a * x@b))
 })
